"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AWSFoundationalSecurityBestPracticesChecker = void 0;
const core_1 = require("@aws-cdk/core");
const aws_iam_1 = require("@aws-cdk/aws-iam");
const aws_lambda_1 = require("@aws-cdk/aws-lambda");
/**
 * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html
 */
class AWSFoundationalSecurityBestPracticesChecker {
    /**
     * Initialise a new Checker. All values in the configuration default to true if not provided.
     * @param config
     */
    constructor(config = {
        iam: { fullAdmin: true, wildcardServiceActions: true },
        lambda: { supportedRuntimes: true },
    }) {
        this.supportedLambdaRuntimes = [
            aws_lambda_1.Runtime.NODEJS_14_X,
            aws_lambda_1.Runtime.NODEJS_12_X,
            aws_lambda_1.Runtime.NODEJS_10_X,
            aws_lambda_1.Runtime.PYTHON_3_8,
            aws_lambda_1.Runtime.PYTHON_3_7,
            aws_lambda_1.Runtime.PYTHON_3_6,
            aws_lambda_1.Runtime.RUBY_2_7,
            aws_lambda_1.Runtime.RUBY_2_5,
            aws_lambda_1.Runtime.JAVA_11,
            aws_lambda_1.Runtime.JAVA_8,
            aws_lambda_1.Runtime.JAVA_8_CORRETTO,
            aws_lambda_1.Runtime.GO_1_X,
            aws_lambda_1.Runtime.DOTNET_CORE_3_1,
            aws_lambda_1.Runtime.DOTNET_CORE_2_1,
        ];
        this.Config = config;
    }
    visit(node) {
        if (node instanceof aws_iam_1.CfnPolicy) {
            this.checkIAMPolicyCompliance(node);
        }
        else if (node instanceof aws_lambda_1.CfnFunction) {
            this.checkLamdaCompliance(node);
        }
    }
    checkLamdaCompliance(node) {
        var _a, _b;
        this.checkLambdaPublicAccess(node);
        if ((_b = (_a = this.Config.lambda) === null || _a === void 0 ? void 0 : _a.supportedRuntimes) !== null && _b !== void 0 ? _b : true) {
            this.checkLambdaSupportedRuntime(node);
        }
    }
    /**
     * [Lambda.1] Lambda function policies should prohibit public access
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-lambda-1
     */
    checkLambdaPublicAccess(node) {
        // TODO: This is complicated...
    }
    /**
     * [Lambda.2] Lambda functions should use supported runtimes
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-lambda-2
     */
    checkLambdaSupportedRuntime(node) {
        if (!this.supportedLambdaRuntimes.some((r) => r.name === node.runtime)) {
            core_1.Annotations.of(node).addError("[Lambda.2] Lambda functions should use supported runtimes");
        }
    }
    checkIAMPolicyCompliance(node) {
        var _a, _b, _c, _d;
        if ((_b = (_a = this.Config.iam) === null || _a === void 0 ? void 0 : _a.fullAdmin) !== null && _b !== void 0 ? _b : true) {
            this.checkIAMPolicyFullAdmin(node);
        }
        if ((_d = (_c = this.Config.iam) === null || _c === void 0 ? void 0 : _c.wildcardServiceActions) !== null && _d !== void 0 ? _d : true) {
            this.checkIAMPolicyServiceWildcards(node);
        }
    }
    /**
     * [IAM.1] IAM policies should not allow full "*" administrative privileges.
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-iam-1
     */
    checkIAMPolicyFullAdmin(node) {
        if (node.policyDocument.hasOwnProperty("statements")) {
            const statements = node.policyDocument.statements;
            if (statements.some((s) => s.effect === aws_iam_1.Effect.ALLOW &&
                s.action.some((a) => a === "*") &&
                s.resource.some((r) => r === "*"))) {
                core_1.Annotations.of(node).addError('[IAM.1] IAM policies should not allow full "*" administrative privileges.');
            }
        }
    }
    /**
     * [IAM.21] IAM customer managed policies that you create should not allow wildcard actions for services
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-iam-21
     */
    checkIAMPolicyServiceWildcards(node) {
        if (node.policyDocument.hasOwnProperty("statements")) {
            const statements = node.policyDocument.statements;
            if (statements.some((s) => s.effect === aws_iam_1.Effect.ALLOW &&
                s.action.some((a) => a.endsWith(":*")))) {
                core_1.Annotations.of(node).addError("[IAM.21] IAM customer managed policies that you create should not allow wildcard actions for services.");
            }
        }
    }
}
exports.AWSFoundationalSecurityBestPracticesChecker = AWSFoundationalSecurityBestPracticesChecker;
